use serde::{Deserialize, Serialize};

#[derive(Debug, PartialEq, Eq, Clone, Deserialize)]
pub struct Request {
    /// Game object describing the game being played.
    game: Game,
    /// Turn number for this move.
    turn: u32,
    /// Board object describing the initial state of the game board.
    board: Board,
    /// Battlesnake Object describing your Battlesnake.
    you: Battlesnake,
}

#[derive(Debug, PartialEq, Eq, Clone, Deserialize)]
pub struct Game {
    /// A unique identifier for this Game.
    id: String,
    /// Information about the ruleset being used to run this Game.
    ruleset: Ruleset,
    /// The name of the map being played on.
    map: String,
    /// How much time your snake has to respond to requests for this Game.
    timeout: u16,
    /// The source of this Game.
    /// One of:
    /// - tournament
    /// - league
    /// - arena
    /// - challenge
    /// - custom
    source: String,
}

#[derive(Debug, PartialEq, Eq, Clone, Deserialize)]
pub struct Ruleset {
    /// Name of the ruleset being used to run this game.
    name: String,
    /// The release version of the Rules module used in this game.
    version: String,
    /// A collection of specific settings being used by the current game that control how the rules
    /// are applied.
    settings: Settings,
}

#[derive(Debug, PartialEq, Eq, Clone, Copy, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct Settings {
    /// Percentage chance of spawning a new food every round.
    food_spawn_chance: u8,
    /// Minimum food to keep on the board every turn.
    minimum_food: u16,
    /// Health damage a snake will take when ending its turn in a hazard. This stacks on top of the
    /// regular 1 damage a snake takes per turn.
    hazard_damage_per_turn: u8,
    /// Settings for the royale game mode
    royale: RoyaleSettings,
}

#[derive(Debug, PartialEq, Eq, Clone, Copy, Deserialize)]
#[serde(rename_all = "camelCase")]
pub struct RoyaleSettings {
    /// The number of turns between generating new hazards (shrinking the safe board space).
    shrink_every_n_turns: u8,
}

#[derive(Debug, PartialEq, Eq, Clone, Deserialize)]
pub struct Board {
    /// The number of rows in the y-axis of the game board.
    height: u16,
    /// The number of rows in the x-axis of the game board.
    width: u16,
    /// Array of coordinates representing food locations on the game board.
    food: Vec<Coord>,
    /// Array of coordinates representing hazardous locations on the game board.
    hazards: Vec<Coord>,
    /// Array of Battlesnake objects representing all Battlesnakes remaining on the game board
    /// (including yourself if you haven't been eliminated).
    snakes: Vec<Battlesnake>,
}

#[derive(Debug, PartialEq, Eq, Clone, Copy, Deserialize)]
pub struct Coord {
    x: u16,
    y: u16,
}

#[derive(Debug, PartialEq, Eq, Clone, Deserialize)]
pub struct Battlesnake {
    /// Unique identifier for this Battlesnake in the context of the current Game.
    id: String,
    /// Name given to this Battlesnake by its author
    name: String,
    /// Health value of this Battlesnake, between 0 and 100
    health: u8,
    /// Array of coordinates representing the Battlesnake's location on the game board.
    /// This array is ordered from head to tail.
    body: Vec<Coord>,
    /// The previous response time of this Battlesnake, in milliseconds.
    /// If the Battlesnake timed out and failed to respond, the game timeout will be returned
    latency: String,
    /// Coordinates for this Battlesnake's head.
    /// Equivalent to the first element of the body array.
    head: Coord,
    /// Length of this Battlesnake from head to tail.
    /// Equivalent to the length of the body array.
    length: u32,
    /// Message shouted by this Battlesnake on the previous turn
    shout: String,
    /// The squad that the Battlesnake belongs to.
    /// Used to identify squad members in Squad Mode games.
    squad: String,
}

#[derive(Debug, PartialEq, Eq, Clone, Serialize)]
pub struct Response {
    /// Your Battlesnake's move for this turn.
    #[serde(rename = "move")]
    pub direction: Direction,
    /// An optional message sent to all other Battlesnakes on the next turn.
    /// Must be 256 characters or less.
    pub shout: Option<String>,
}

#[derive(Debug, PartialEq, Eq, Clone, Copy, Serialize)]
pub enum Direction {
    /// Move in positive y direction
    Up,
    /// Move in negative y direction
    Down,
    /// Move in negative x direction
    Left,
    /// Move in positive x direction
    Right,
}
